@@ -2,12 +2,25 @@
 
 /* #define DEBUG Don't even think defining it:) */
 
+#include <mhash.h>
+#include <mcrypt.h>
+
+#ifndef LIBMCRYPT_22
+# if MCRYPT_API_VERSION < 20000222
+#  error "Your mcrypt.h header file is older than the library"
+# endif
+#endif
+
 #ifdef STDC_HEADERS
 # include <string.h>
 # include <stdlib.h>
 # include <stdio.h>
 #endif
 
+#ifdef HAVE_CTYPE_H
+# include <ctype.h>
+#endif
+
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
@@ -75,17 +88,31 @@
 
 /* defines */
 #define SALT_SIZE 20
-#define RESERVED 20
 
-#define MD5HASH 3
-#define SHA1HASH 2
+#define LIBHANDLED 2
 #define BITS4 1
 #define BITS8 0
 
 #define TRUE 1
 #define FALSE 0
 
-#define NO_TYPE 999
+#define HASH_SIZE 10
+
+#define BUFFER_SIZE 1024
+#define KEYMODE_SIZE 20
+#define ALGORITHM_SIZE 15
+#define MODE_SIZE 10
+#define IV_SIZE 64
+
+#ifdef LIBMCRYPT_22
+# define DEFAULT_ALGORITHM "twofish-256"
+#else
+# define DEFAULT_ALGORITHM "twofish"
+#endif
+
+#define DEFAULT_MODE "cbc"
+#define DEFAULT_KEYMODE "mcrypt-sha1"
+#define HASH_ALGORITHM "crc32"
 
 #define ENCRYPT 0
 #define DECRYPT 1
