@@ -1,17 +1,17 @@
 #define MAX_KEY_LEN 513
 
-char * get_password(int numofchar,int bitmode,int mode,int type, unsigned int *len, void * salt);
+char * get_password(int numofchar, int mode, unsigned int *len, void * salt);
 int show_mode(int ende, int stype,int smode,char *output,int bitmode);
 char** read_key_file(char * file, int* num);
 char *get_cfile(int uid, char*);
-
-int is_block_algorithm_local(int algorithm);
-char* get_algorithms_name_local(int);
+void Bzero(void *s, size_t n);
+void _tolow(char *str, int size);
 
 char *my_getpass();
-
-int check_file_head(char * filename, unsigned int *algorithm, unsigned int *mode, unsigned int *bit7_flag,unsigned int stream_flag);
-int write_file_head(FILE* filedes, unsigned int algorithm, unsigned int mode, unsigned int bit7_flag, void* salt);
+int check_file_head(FILE *fstream, char *algorithm, char* mode, char* keymode, int *keysize, void* salt, int* salt_size);
+void* read_iv(FILE *fstream, int ivsize);
+int write_file_head(FILE * filedes, char* algorithm, char* mode, char* keymode, int* keysize, void *salt, int salt_size);
+int write_iv(FILE * filedes, void* IV, int ivsize);
 
 void cleanUp();
 void shandler(int signal);
