@@ -112,18 +112,25 @@ void gaa_help()
 	__gaa_helpsingle('p', "bzip2", "", "Compress before encryption.");
 	__gaa_helpsingle('n', "nolock", "", "Do not lock files.");
 	__gaa_helpsingle(0, "echo", "", "Echo asterisks.");
+	__gaa_helpsingle(0, "noiv", "", "Do not use IV.");
+	__gaa_helpsingle(0, "nodelete", "", "Do not delete the output file if decryption failed.");
 	__gaa_helpsingle(0, "flush", "", "Immediately flush the output");
 	__gaa_helpsingle('l', "doublecheck", "", "Double check passwords.");
 	__gaa_helpsingle('d', "decrypt", "", "decrypts.");
-	__gaa_helpsingle('x', "hex", "", "Keyword is in hex.");
-	__gaa_helpsingle('o', "keymode", """kmode"" ", "keyword mode (4, 8 or hash).");
+	__gaa_helpsingle('s', "keysize", """integer"" ", "The algorithm's key size.");
+	__gaa_helpsingle(0, "algorithms_directory", """algorithms_directory"" ", "algorithms directory.");
+	__gaa_helpsingle(0, "modes_directory", """modes_directory"" ", "modes directory.");
+	__gaa_helpsingle('o', "keymode", """kmode"" ", "keyword mode.");
 	__gaa_helpsingle('u', "unlink", "", "unlink the input file after""encryption/decryption.");
 	__gaa_helpsingle('f', "keyfile", """keyfile"" ", "file to read the keyword from.");
 	__gaa_helpsingle('c', "config", """config"" ", "Use configuration file FILE.");
 	__gaa_helpsingle('a', "algorithm", """algorithm"" ", "specify the encryption and decryption algorithm.""Legal parameters are: 3way, des, 3des, gost, tea,""safer64, safer128, cast128, crypt, twofish""and blowfish. (Default is blowfish)");
 	__gaa_helpsingle('k', "key", """key1 key2...keyN"" ", "specify the key(s)");
 	__gaa_helpsingle('m', "mode", """mode"" ", "specify the encryption and decryption mode.""CBC, ECB or CFB. CBC is the default.");
-	__gaa_helpsingle(0, "list", "", "Print a list of the supported algorithms and modes.");
+	__gaa_helpsingle('h', "hash", """hash"" ", "specify the hash algorithm to be used.");
+	__gaa_helpsingle(0, "list", "", "Print a list of the supported algorithms  and modes.");
+	__gaa_helpsingle(0, "keymodeslist", "", "Print a list of the supported key modes.");
+	__gaa_helpsingle(0, "hashlist", "", "Print a list of the supported hash algorithms.");
 	__gaa_helpsingle('h', "help", "", "prints this help");
 	__gaa_helpsingle('v', "version", "", "prints the version number");
 	__gaa_helpsingle('L', "license", "", "displays license information.");
@@ -141,40 +148,50 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 77 "mcrypt.gaa"
+#line 96 "mcrypt.gaa"
 	char **input;
-#line 76 "mcrypt.gaa"
+#line 95 "mcrypt.gaa"
 	int size;
-#line 67 "mcrypt.gaa"
+#line 86 "mcrypt.gaa"
+	char *hash;
+#line 82 "mcrypt.gaa"
 	char *mode;
-#line 64 "mcrypt.gaa"
+#line 77 "mcrypt.gaa"
 	char **keys;
-#line 63 "mcrypt.gaa"
-	int keysize;
-#line 57 "mcrypt.gaa"
+#line 76 "mcrypt.gaa"
+	int keylen;
+#line 70 "mcrypt.gaa"
 	char *algorithm;
-#line 54 "mcrypt.gaa"
+#line 67 "mcrypt.gaa"
 	char *config_file;
-#line 53 "mcrypt.gaa"
+#line 66 "mcrypt.gaa"
 	int config;
-#line 50 "mcrypt.gaa"
+#line 63 "mcrypt.gaa"
 	char *keyfile;
-#line 46 "mcrypt.gaa"
+#line 59 "mcrypt.gaa"
 	int unlink_flag;
-#line 43 "mcrypt.gaa"
+#line 56 "mcrypt.gaa"
 	char *kmode;
-#line 40 "mcrypt.gaa"
-	int hexflag;
-#line 37 "mcrypt.gaa"
+#line 52 "mcrypt.gaa"
+	char *modes_directory;
+#line 49 "mcrypt.gaa"
+	char *algorithms_directory;
+#line 46 "mcrypt.gaa"
+	int keysize;
+#line 43 "mcrypt.gaa"
 	int ed_specified;
-#line 36 "mcrypt.gaa"
+#line 42 "mcrypt.gaa"
 	int ein;
-#line 35 "mcrypt.gaa"
+#line 41 "mcrypt.gaa"
 	int din;
-#line 32 "mcrypt.gaa"
+#line 38 "mcrypt.gaa"
 	int double_check;
-#line 29 "mcrypt.gaa"
+#line 35 "mcrypt.gaa"
 	int flush;
+#line 31 "mcrypt.gaa"
+	int nodelete;
+#line 28 "mcrypt.gaa"
+	int noiv;
 #line 25 "mcrypt.gaa"
 	int noecho;
 #line 22 "mcrypt.gaa"
@@ -245,31 +262,38 @@ int gaa_error = 0;
 #define GAA_MULTIPLE_OPTION     3
 
 #define GAA_REST                0
-#define GAA_NB_OPTION           24
+#define GAA_NB_OPTION           31
 #define GAAOPTID_license	1
 #define GAAOPTID_version	2
 #define GAAOPTID_help	3
-#define GAAOPTID_list	4
-#define GAAOPTID_mode	5
-#define GAAOPTID_key	6
-#define GAAOPTID_algorithm	7
-#define GAAOPTID_config	8
-#define GAAOPTID_keyfile	9
-#define GAAOPTID_unlink	10
-#define GAAOPTID_keymode	11
-#define GAAOPTID_hex	12
-#define GAAOPTID_decrypt	13
-#define GAAOPTID_doublecheck	14
-#define GAAOPTID_flush	15
-#define GAAOPTID_echo	16
-#define GAAOPTID_nolock	17
-#define GAAOPTID_bzip2	18
-#define GAAOPTID_gzip	19
-#define GAAOPTID_bare	20
-#define GAAOPTID_random	21
-#define GAAOPTID_force	22
-#define GAAOPTID_quiet	23
-#define GAAOPTID_verbose	24
+#define GAAOPTID_hashlist	4
+#define GAAOPTID_keymodeslist	5
+#define GAAOPTID_list	6
+#define GAAOPTID_hash	7
+#define GAAOPTID_mode	8
+#define GAAOPTID_key	9
+#define GAAOPTID_algorithm	10
+#define GAAOPTID_config	11
+#define GAAOPTID_keyfile	12
+#define GAAOPTID_unlink	13
+#define GAAOPTID_keymode	14
+#define GAAOPTID_modes_directory	15
+#define GAAOPTID_algorithms_directory	16
+#define GAAOPTID_keysize	17
+#define GAAOPTID_decrypt	18
+#define GAAOPTID_doublecheck	19
+#define GAAOPTID_flush	20
+#define GAAOPTID_nodelete	21
+#define GAAOPTID_noiv	22
+#define GAAOPTID_echo	23
+#define GAAOPTID_nolock	24
+#define GAAOPTID_bzip2	25
+#define GAAOPTID_gzip	26
+#define GAAOPTID_bare	27
+#define GAAOPTID_random	28
+#define GAAOPTID_force	29
+#define GAAOPTID_quiet	30
+#define GAAOPTID_verbose	31
 
 #line 168 "gaa.skel"
 
@@ -452,6 +476,12 @@ float gaa_getfloat(char *arg)
 }
 /* option structures */
 
+struct GAAOPTION_hash 
+{
+	char* arg1;
+	int size1;
+};
+
 struct GAAOPTION_mode 
 {
 	char* arg1;
@@ -487,6 +517,24 @@ struct GAAOPTION_keymode
 	char* arg1;
 	int size1;
 };
+
+struct GAAOPTION_modes_directory 
+{
+	char* arg1;
+	int size1;
+};
+
+struct GAAOPTION_algorithms_directory 
+{
+	char* arg1;
+	int size1;
+};
+
+struct GAAOPTION_keysize 
+{
+	int arg1;
+	int size1;
+};
 #define GAA_REST_EXISTS
 
 struct GAAREST
@@ -524,23 +572,30 @@ int gaa_get_option_num(char *str, int status)
     switch(status)
         {
         case GAA_LETTER_OPTION:
+			GAA_CHECK1STR("h", GAAOPTID_hash);
 			GAA_CHECK1STR("m", GAAOPTID_mode);
 			GAA_CHECK1STR("k", GAAOPTID_key);
 			GAA_CHECK1STR("a", GAAOPTID_algorithm);
 			GAA_CHECK1STR("c", GAAOPTID_config);
 			GAA_CHECK1STR("f", GAAOPTID_keyfile);
 			GAA_CHECK1STR("o", GAAOPTID_keymode);
+			GAA_CHECK1STR("", GAAOPTID_modes_directory);
+			GAA_CHECK1STR("", GAAOPTID_algorithms_directory);
+			GAA_CHECK1STR("s", GAAOPTID_keysize);
         case GAA_MULTIPLE_OPTION:
 #line 375 "gaa.skel"
 			GAA_CHECK1STR("L", GAAOPTID_license);
 			GAA_CHECK1STR("v", GAAOPTID_version);
 			GAA_CHECK1STR("h", GAAOPTID_help);
+			GAA_CHECK1STR("", GAAOPTID_hashlist);
+			GAA_CHECK1STR("", GAAOPTID_keymodeslist);
 			GAA_CHECK1STR("", GAAOPTID_list);
 			GAA_CHECK1STR("u", GAAOPTID_unlink);
-			GAA_CHECK1STR("x", GAAOPTID_hex);
 			GAA_CHECK1STR("d", GAAOPTID_decrypt);
 			GAA_CHECK1STR("l", GAAOPTID_doublecheck);
 			GAA_CHECK1STR("", GAAOPTID_flush);
+			GAA_CHECK1STR("", GAAOPTID_nodelete);
+			GAA_CHECK1STR("", GAAOPTID_noiv);
 			GAA_CHECK1STR("", GAAOPTID_echo);
 			GAA_CHECK1STR("n", GAAOPTID_nolock);
 			GAA_CHECK1STR("p", GAAOPTID_bzip2);
@@ -557,7 +612,10 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("license", GAAOPTID_license);
 			GAA_CHECKSTR("version", GAAOPTID_version);
 			GAA_CHECKSTR("help", GAAOPTID_help);
+			GAA_CHECKSTR("hashlist", GAAOPTID_hashlist);
+			GAA_CHECKSTR("keymodeslist", GAAOPTID_keymodeslist);
 			GAA_CHECKSTR("list", GAAOPTID_list);
+			GAA_CHECKSTR("hash", GAAOPTID_hash);
 			GAA_CHECKSTR("mode", GAAOPTID_mode);
 			GAA_CHECKSTR("key", GAAOPTID_key);
 			GAA_CHECKSTR("algorithm", GAAOPTID_algorithm);
@@ -565,10 +623,14 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("keyfile", GAAOPTID_keyfile);
 			GAA_CHECKSTR("unlink", GAAOPTID_unlink);
 			GAA_CHECKSTR("keymode", GAAOPTID_keymode);
-			GAA_CHECKSTR("hex", GAAOPTID_hex);
+			GAA_CHECKSTR("modes_directory", GAAOPTID_modes_directory);
+			GAA_CHECKSTR("algorithms_directory", GAAOPTID_algorithms_directory);
+			GAA_CHECKSTR("keysize", GAAOPTID_keysize);
 			GAA_CHECKSTR("decrypt", GAAOPTID_decrypt);
 			GAA_CHECKSTR("doublecheck", GAAOPTID_doublecheck);
 			GAA_CHECKSTR("flush", GAAOPTID_flush);
+			GAA_CHECKSTR("nodelete", GAAOPTID_nodelete);
+			GAA_CHECKSTR("noiv", GAAOPTID_noiv);
 			GAA_CHECKSTR("echo", GAAOPTID_echo);
 			GAA_CHECKSTR("nolock", GAAOPTID_nolock);
 			GAA_CHECKSTR("bzip2", GAAOPTID_bzip2);
@@ -590,12 +652,16 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 {
     int OK = 0;
     int gaa_last_non_option;
+	struct GAAOPTION_hash GAATMP_hash;
 	struct GAAOPTION_mode GAATMP_mode;
 	struct GAAOPTION_key GAATMP_key;
 	struct GAAOPTION_algorithm GAATMP_algorithm;
 	struct GAAOPTION_config GAATMP_config;
 	struct GAAOPTION_keyfile GAATMP_keyfile;
 	struct GAAOPTION_keymode GAATMP_keymode;
+	struct GAAOPTION_modes_directory GAATMP_modes_directory;
+	struct GAAOPTION_algorithms_directory GAATMP_algorithms_directory;
+	struct GAAOPTION_keysize GAATMP_keysize;
 
 #line 393 "gaa.skel"
 #ifdef GAA_REST_EXISTS
@@ -618,30 +684,54 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
     {
 	case GAAOPTID_license:
 	OK = 0;
-#line 74 "mcrypt.gaa"
+#line 93 "mcrypt.gaa"
 { mcrypt_license(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_version:
 	OK = 0;
-#line 73 "mcrypt.gaa"
+#line 92 "mcrypt.gaa"
 { mcrypt_version(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_help:
 	OK = 0;
-#line 72 "mcrypt.gaa"
+#line 91 "mcrypt.gaa"
 { usage(); exit(0); ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_hashlist:
+	OK = 0;
+#line 90 "mcrypt.gaa"
+{ print_hashlist(); exit(0); ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_keymodeslist:
+	OK = 0;
+#line 89 "mcrypt.gaa"
+{ print_keylist(); exit(0); ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_list:
 	OK = 0;
-#line 71 "mcrypt.gaa"
+#line 88 "mcrypt.gaa"
 { print_list(); exit(0); ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_hash:
+	OK = 0;
+		GAA_TESTMOREARGS;
+		GAA_FILL(GAATMP_hash.arg1, gaa_getstr, GAATMP_hash.size1);
+		gaa_index++;
+#line 87 "mcrypt.gaa"
+{ gaaval->hash = GAATMP_hash.arg1 ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_mode:
@@ -649,7 +739,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_mode.arg1, gaa_getstr, GAATMP_mode.size1);
 		gaa_index++;
-#line 68 "mcrypt.gaa"
+#line 83 "mcrypt.gaa"
 { gaaval->mode = GAATMP_mode.arg1 ;};
 
 		return GAA_OK;
@@ -657,8 +747,8 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 	case GAAOPTID_key:
 	OK = 0;
 		GAA_LIST_FILL(GAATMP_key.arg1, gaa_getstr, char*, GAATMP_key.size1);
-#line 65 "mcrypt.gaa"
-{ gaaval->keys = GAATMP_key.arg1; gaaval->keysize = GAATMP_key.size1 ;};
+#line 78 "mcrypt.gaa"
+{ gaaval->keys = GAATMP_key.arg1; gaaval->keylen = GAATMP_key.size1 ;};
 
 		return GAA_OK;
 		break;
@@ -667,7 +757,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithm.arg1, gaa_getstr, GAATMP_algorithm.size1);
 		gaa_index++;
-#line 58 "mcrypt.gaa"
+#line 71 "mcrypt.gaa"
 { gaaval->algorithm = GAATMP_algorithm.arg1 ;};
 
 		return GAA_OK;
@@ -677,7 +767,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_config.arg1, gaa_getstr, GAATMP_config.size1);
 		gaa_index++;
-#line 55 "mcrypt.gaa"
+#line 68 "mcrypt.gaa"
 { gaaval->config=1; if (GAATMP_config.size1==1) gaaval->config_file = GAATMP_config.arg1 ;};
 
 		return GAA_OK;
@@ -687,14 +777,14 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keyfile.arg1, gaa_getstr, GAATMP_keyfile.size1);
 		gaa_index++;
-#line 51 "mcrypt.gaa"
+#line 64 "mcrypt.gaa"
 { gaaval->keyfile = GAATMP_keyfile.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_unlink:
 	OK = 0;
-#line 47 "mcrypt.gaa"
+#line 60 "mcrypt.gaa"
 { gaaval->unlink_flag = 1 ;};
 
 		return GAA_OK;
@@ -704,37 +794,74 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keymode.arg1, gaa_getstr, GAATMP_keymode.size1);
 		gaa_index++;
-#line 44 "mcrypt.gaa"
+#line 57 "mcrypt.gaa"
 { gaaval->kmode = GAATMP_keymode.arg1 ;};
 
 		return GAA_OK;
 		break;
-	case GAAOPTID_hex:
+	case GAAOPTID_modes_directory:
 	OK = 0;
-#line 41 "mcrypt.gaa"
-{ gaaval->hexflag = 1 ;};
+		GAA_TESTMOREARGS;
+		GAA_FILL(GAATMP_modes_directory.arg1, gaa_getstr, GAATMP_modes_directory.size1);
+		gaa_index++;
+#line 53 "mcrypt.gaa"
+{ gaaval->modes_directory = GAATMP_modes_directory.arg1 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_algorithms_directory:
+	OK = 0;
+		GAA_TESTMOREARGS;
+		GAA_FILL(GAATMP_algorithms_directory.arg1, gaa_getstr, GAATMP_algorithms_directory.size1);
+		gaa_index++;
+#line 50 "mcrypt.gaa"
+{ gaaval->algorithms_directory = GAATMP_algorithms_directory.arg1 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_keysize:
+	OK = 0;
+		GAA_TESTMOREARGS;
+		GAA_FILL(GAATMP_keysize.arg1, gaa_getint, GAATMP_keysize.size1);
+		gaa_index++;
+#line 47 "mcrypt.gaa"
+{ gaaval->keysize = GAATMP_keysize.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_decrypt:
 	OK = 0;
-#line 38 "mcrypt.gaa"
+#line 44 "mcrypt.gaa"
 { gaaval->ed_specified=1; gaaval->din = 1; gaaval->ein=0; ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_doublecheck:
 	OK = 0;
-#line 33 "mcrypt.gaa"
+#line 39 "mcrypt.gaa"
 { gaaval->double_check = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_flush:
 	OK = 0;
-#line 30 "mcrypt.gaa"
+#line 36 "mcrypt.gaa"
 { gaaval->flush = 1 ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_nodelete:
+	OK = 0;
+#line 32 "mcrypt.gaa"
+{ gaaval->nodelete = 1 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_noiv:
+	OK = 0;
+#line 29 "mcrypt.gaa"
+{ gaaval->noiv = 1 ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_echo:
@@ -802,7 +929,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		break;
 	case GAA_REST:
 		GAA_OPTIONALLIST_FILL(GAAREST_tmp.arg1, gaa_getstr, char*, GAAREST_tmp.size1);
-#line 78 "mcrypt.gaa"
+#line 97 "mcrypt.gaa"
 { gaaval->input = GAAREST_tmp.arg1; gaaval->size = GAAREST_tmp.size1 ;};
 
 		return GAA_OK;
@@ -830,13 +957,15 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     if(inited == 0)
     {
 
-#line 80 "mcrypt.gaa"
+#line 99 "mcrypt.gaa"
 { gaaval->force=0; gaaval->quiet=1; gaaval->real_random_flag=0; gaaval->config=0;
-       gaaval->bare_flag = 0; gaaval->din = 0; gaaval->ein=1; gaaval->hexflag = 0; gaaval->unlink_flag = 0;
-       gaaval->kmode = NULL; gaaval->keysize=0; gaaval->gzipflag=0; gaaval->bzipflag=0;
-       gaaval->keyfile=NULL; gaaval->algorithm=NULL; gaaval->keys=NULL;
+       gaaval->bare_flag = 0; gaaval->din = 0; gaaval->ein=1; gaaval->unlink_flag = 0;
+       gaaval->kmode = NULL; gaaval->keylen=0; gaaval->gzipflag=0; gaaval->bzipflag=0;
+       gaaval->keyfile=NULL; gaaval->algorithm=NULL; gaaval->keys=NULL; gaaval->noiv=0;
        gaaval->config_file=NULL; gaaval->mode=NULL; gaaval->input=NULL; gaaval->ed_specified=0;
-       gaaval->double_check=0; gaaval->nolock=0; gaaval->noecho=1; gaaval->flush=0; ;};
+       gaaval->double_check=0; gaaval->nolock=0; gaaval->noecho=1; gaaval->flush=0; gaaval->keysize=0;
+       gaaval->algorithms_directory=NULL; gaaval->modes_directory=NULL; gaaval->nodelete=0;
+       gaaval->hash=NULL ;};
 
     }
     inited = 1;
